/**
 * Contains team names, problems and submission history
 **/

#include <string>
#include <map>
#include <set>
#include <fstream>
#include <sstream>
#include <ctype.h>
#include "db.h"

using namespace std;

/** These are extern in db.h **/
map< string, Team > g_teams;
set< string > g_probs;

void initdb( string teamfile, string probfile, string logfile )
{
    // Read the team file
    ifstream tin( teamfile.c_str(), ios::in );
    string line;
    while( getline( tin, line ) )
    {
        if( !line.size() || line[0] == '#' ) continue;
        string n, fn;
        istringstream sin( line );
        sin >> n;
        while( sin.peek() == ' ' || sin.peek() == '\t' ) sin.get();
        getline( sin, fn );
        g_teams[n] = Team( n, fn );
    }
    tin.close();

    // Read problem file
    ifstream pin( probfile.c_str(), ios::in );
    while( getline( pin, line ) )
    {
        if( !line.size() || line[0] == '#' ) continue;
        istringstream lin( line );
        while( lin >> line ) g_probs.insert( line );
    }

    // Read the log to catch up on recent history
    ifstream lin( logfile.c_str(), ios::in );
    while( getline( lin, line ) )
    {
        if( !line.size() || line[0] == '#' ) continue;
        string name;
        istringstream sin( line );
        sin >> name;
        // Line of the form: <team> submits <prob> {yes | no <comment>}
        if( g_teams.count( name ) )
        {
            string prob; sin >> prob >> prob;
            string ans; sin >> ans;
            if( ans == "yes" )
                g_teams[name].solved.insert( prob );
            else if( !g_teams[name].solved.count( prob ) )
                g_teams[name].tries[prob]++;
        }
    }
}

bool teamExists( string team )
{
    return g_teams.count( team );
}

bool probExists( string prob )
{
    return g_probs.count( prob );
}

bool langExists( string lang )
{
    return( lang == "c" || lang == "c++" );
}

const char *compileCmd( string lang )
{
    //FIXME: works only for c and c++
    if( lang == "c" ) return "gcc";
    return "g++";
}

const char *executeCmd( string lang )
{
    //FIXME: works only for c and c++
    return "a.out";
}

const char *sourceSuffix( string lang )
{
    //FIXME: works only for c and c++
    if( lang == "c" ) return "c";
    else return "cpp";
}

