#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <string.h>
#include <string>
#include <fstream>
#include <fcntl.h>
#include <stdio.h>

using namespace std;

void wakeup( int sig )
{
    // Reinstall handler
    signal( SIGHUP, &wakeup );

    // Turn off signal processing
    sigset_t mask_set;
    sigset_t old_set;
    sigfillset( &mask_set );
    sigprocmask( SIG_SETMASK, &mask_set, &old_set );

    printf( "Caught SIGHUP\n" );
    //FIXME: add a submission to the queue
}

/**
 * This is the Judy daemon. DON'T RUN AS ROOT!
 * It sleeps and waits for SIGHUPs. When signaled (by the
 * Perl script), it processes the submission.
 * Expects 2 command line arguments:
 *     - The "incoming" directory, where submissions are stored
 *     - The pid file name
 *     - Port to open
 *     - Judy config file
 **/
int main( int argc, char **argv )
{
    if( argc != 5 )
    {
        printf( "Start judyd through \"judy\"\n" );
        return -1;
    }

    // Check that the "incoming" dir exists
    DIR *dir = opendir( argv[1] );
    if( !dir )
    {
        fprintf( stderr, "Can not open directory %s\n", argv[1] );
        return 1;
    }
    closedir( dir );

    // If the pid file exists, kill the other copy of judyd
    int pidf = open( argv[2], O_RDONLY );
    int pid;
    char buf[16];
    if( pidf != -1 )
    {
        if( read( pidf, buf, 15 ) )
        {
            sscanf( buf, "%d", &pid );
            kill( pid, SIGKILL );
        }
        close( pidf );
    }

    // Write pid to pid file
    pidf = open( argv[2], O_WRONLY | O_CREAT | O_TRUNC );
    if( pidf == -1 )
    {
        fprintf( stderr, "Can not create pid file %s\n", argv[2] );
        return 2;
    }
    memset( buf, 0, sizeof( buf ) );
    sprintf( buf, "%d", getpid() + 1 );             //XXX: tell me how to do this right
    if( !write( pidf, buf, strlen( buf ) ) )
    {
        fprintf( stderr, "Can not write to pid file %s\n", argv[2] );
        return 3;
    }
    close( pidf );

    // Open a socket for listening
    int s = socket( AF_INET, SOCK_STREAM, 0 );
    if( s == -1 )
    {
        fprintf( stderr, "Can not create a TCP socket.\n" );
        return 4;
    }

    // Bind the socket to the given port
    short port = atoi( argv[3] );
    struct sockaddr_in local;
    memset( ( char* )&local, sizeof( local ), 0 );
    local.sin_family = AF_INET;
    local.sin_port = htons( port );
    local.sin_addr.s_addr = htonl( INADDR_ANY );
    if( bind( s, ( struct sockaddr* )&local, sizeof( local ) ) == -1 )
    {
        fprintf( stderr, "Can not bind socket to port %d\n", port );
        return 5;
    }

    // Prepare socket 's' for listening
    listen( s, 32 );

    // All is fine, start the daemon
    printf( "Judy Daemon is alive.\n" );
    printf( "    Incoming directory is \"%s\"\n", argv[1] );
    printf( "    Pid file \"%s\" has been created\n", argv[2] );

    // Install SIGHUP handler
    signal( SIGHUP, &wakeup );

    // To prevent fork()ed process orphanage (<defunct> business)
    signal( SIGCHLD, SIG_IGN );

    // Go to background
    daemon( 1, 1 );

    srand( getpid() );

    // Listen for connections to 's'
    struct sockaddr_in remote;
    while( 1 )
    {
        unsigned int remoteSize;
        int sock = accept( s, ( struct sockaddr* )&remote, &remoteSize );

        // Read parameters from the socket
        char c;
        string teamID, prob, lang;
        while( read( sock, &c, 1 ) && c != '\n' ) teamID += c;
        while( read( sock, &c, 1 ) && c != '\n' ) prob += c;
        while( read( sock, &c, 1 ) && c != '\n' ) lang += c;

        // Create code file
        char tmpfile[1024];
        memset( tmpfile, 0, sizeof( tmpfile ) );
        sprintf( tmpfile, "%s/%s-%d", argv[1], teamID.c_str(), rand() & 0xFFFF );
        ofstream fout( tmpfile, ios::out );
        while( read( sock, &c, 1 ) ) fout << c;
        fout.close();

        // Call on Judy to judge this submission
        char cmd[2048];
        memset( cmd, 0, sizeof( cmd ) );
        sprintf( cmd, "./judy %s %s %s %s %s",
            argv[4],            // Config file
            teamID.c_str(),
            prob.c_str(),
            lang.c_str(),
            tmpfile );
        char outbuf[32];
        memset( outbuf, 0, sizeof( outbuf ) );

        // Let a clone do all the hard work
        int pid = fork();
        if( pid == -1 ) return -2;          // God save me from this!
        if( !pid )
        {
            // Service the request and die
            system( cmd );
            return 0;
        }
        close( sock );
    }

    return 0;
}
